Four types of intramolecular potentials are used to describe the molecules: 

## Harmonic bond

The only bond potential implemented is the harmonic bond potential, defined as

```math
V(r_{ij}) = k_0(|r_{ij}| - r_0)^2
```

where $`r_0`$ is the equilibrium length and $`k_0`$ is the spring constant. When such a potential is defined between two atoms (even set to 0), their Coulomb and van der Waals interactions are cancelled. 

## Harmonic angle

The only angle potential implemented is the harmonic angle potential, defined as

```math
V(r_i,r_j,r_k) = k_0(\theta_{ijk} - \theta_0)^2
```

where $`\theta_0`$ is the equilibrium angle, $`k_0`$ is the spring constant and the angle $`\theta_{ijk}`$ is defined so that $`j`$ is the central atom of the angle. When such a potential is defined between three atoms (even set to 0), their Coulomb and van der Waals interactions are cancelled. 


## Dihedral angle

The potential for dihedral angles is

```math
V(r_i,r_j,r_k,r_l) = 0.5*(v_1(1+cos(\phi_{ijkl}))+v_2(1-cos(2\phi_{ijkl}))+v_3(1+cos(3\phi_{ijkl}))+v_4*(1-cos(4\phi_{ijkl})))
```
where $`v_1`$, $`v_2`$, $`v_3`$ and $`v_4`$ are parameters and $`i, j, k, l`$ designate the atoms in the dihedral angle in a consecutive order ($`i`$ is bonded to $`j`$ which is bonded to $`k`$ which is bonded to $`l`$). When such a potential is defined between four atoms (even set to 0), their Coulomb and van der Waals interactions are rescaled by a factor 0.5. 


## Improper angle

The potential for improper angles is

```math
V(r_i,r_j,r_k,r_l) = 0.5*(v_1(1+cos(\phi_{ijkl}))+v_2(1-cos(2\phi_{ijkl}))+v_3(1+cos(3\phi_{ijkl}))+v_4*(1-cos(4\phi_{ijkl})))
```
where $`v_1`$, $`v_2`$, $`v_3`$ and $`v_4`$ are parameters and $`i, j, l`$ designate the atoms in the improper angle which are all bonded to the central atom $`k`$. When such a potential is defined between four atoms (even set to 0), their Coulomb and van der Waals interactions are rescaled by a factor 0.5. 