Units
=====

In MetalWalls, all quantities are normalized to reference quantities. These are expressed in atomic units.

Fundamental atomic units
------------------------

The electron mass, the elementary charge, the reduced Planck's constant and the Coulomb force constants provide the basis for the normalizations in atomic units.

+ Reference mass
  - Name: electron rest mass
  - Symbol: $`m_e`$
  - Value in SI units: $`9.10938291\ 10^{-31}\ \mathrm{kg}`$


+ Reference electric charge
  - Name: elementary charge
  - Symbol: $`e`$
  - Value in SI units: $`1.602176565\ 10^{-19}\ \mathrm{C}`$


+ Reference action
  - Name: reduced Planck's constant
  - Symbol: $`\hbar = h / (2 \pi)`$
  - Value in SI units: $`1.054571726\ 10^{-34}\ \mathrm{J}\cdot\mathrm{s}`$

+ Reference electric constant
  - Name: Coulomb force constant
  - Symbol: $`k_e = 1 / (4 \pi \epsilon_0)`$
  - Value in SI units: $`8.9875517873681\ 10^9\ \mathrm{kg}\cdot \mathrm{m}^3\cdot\mathrm{s}^{-2}\cdot\mathrm{C}^{-2}`$
    
Derived atomic units
--------------------

From the fundamental units we can derived other units. $`k_B`$ is the Boltzmann constant.

+ Reference length
  - Name: Bohr radius
  - Symbol: $`a_0 = 4 \pi \epsilon_0 \hbar^2 / (m_e e^2)`$
  - Value in SI units: $`5.2917721092\ 10^{-11}\ \mathrm{m}`$

+ Reference energy
  - Name: Hartree
  - Symbol: $`E_h = m_e e^4 / ( 4 \pi \epsilon_0 \hbar)^2`$
  - Value in SI units: $`4.35974417\ 10^{-18}\ \mathrm{J}`$

+ Reference time
  - Expression: $`\hbar / E_h`$
  - Value in SI units: $`2.418884326505\ 10^{-17} \mathrm{s}`$

+ Reference velocity
  - Expression: $`a_0 E_h / \hbar`$
  - Value in SI units: $`2.1876912633\ 10^6 \mathrm{m}\cdot\mathrm{s}^{-1}`$

+ Reference force
  - Expression: $`E_h / a_0`$
  - Value in SI units: $`8.2387225\ 10^{-8}\ \mathrm{N}`$

+ Reference temperature
  - Expression: $`E_h / k_B`$
  - Value in SI units: $`3.1577464\ 10^{5} \mathrm{K}`$

+ Reference electric field
  - Expression: $`E_h / (e a_0)`$
  - Value in SI units: $`5.14220652\ 10^{11}\ \mathrm{V}\cdot\mathrm{m}^{-1}`$

+ Reference electric potential
  - Expression: $`E_h / e`$
  - Value in SI units: $`2.721138505\ 10^1\ \mathrm{V}`$    

+ Reference pressure
  - Expression: $`E_h / a_0^3`$
  - Value in SI units: $`2.94219120\ 10^8\ \mathrm{bar}`$    

+ Reference dipole moment
  - Expression: $`E_h a_0`$
  - Value in SI units: $`8.4783536255\ 10^{-30}\ \mathrm{C/m}`$    
